"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
var _react = require("@testing-library/react");
var _vitest = require("vitest");
require("@testing-library/jest-dom");
var _measureText = _interopRequireDefault(require("../measureText"));
var _jsxRuntime = require("@emotion/react/jsx-runtime");
var _div, _div2, _div3, _div4;
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015 - present Instructure, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
const baseStyle = {
  fontSize: '16px',
  fontFamily: 'Arial',
  fontWeight: 'normal',
  fontStyle: 'normal',
  letterSpacing: 'normal'
};
const getNodes = root => Array.from(root.childNodes).filter(node => node.nodeType === Node.ELEMENT_NODE || node.nodeType === Node.TEXT_NODE);
describe('measureText', () => {
  beforeEach(() => {
    _vitest.vi.spyOn(HTMLCanvasElement.prototype, 'getContext').mockImplementation(() => {
      return {
        font: '',
        measureText: _vitest.vi.fn(text => {
          return {
            width: text.length * 10
          };
        })
      };
    });
  });
  afterEach(() => {
    _vitest.vi.restoreAllMocks();
  });
  it('should calculate the width of a single text node correctly', () => {
    const node = document.createElement('div');
    Object.assign(node.style, baseStyle);
    node.textContent = 'Hello';
    const width = (0, _measureText.default)([node]);
    (0, _vitest.expect)(width).toBe(50); // 5 characters * 10 (mocked width per character)
  });
  it('should return zero for empty nodes', () => {
    const node = document.createElement('div');
    Object.assign(node.style, baseStyle);
    node.textContent = '';
    const width = (0, _measureText.default)([node]);
    (0, _vitest.expect)(width).toBe(0);
  });
  it('should calculate text width correctly', () => {
    (0, _react.render)(_div || (_div = (0, _jsxRuntime.jsxs)("div", {
      "data-testid": "stage",
      style: baseStyle,
      children: ["Lorem ipsum ", (0, _jsxRuntime.jsx)("span", {
        style: baseStyle,
        children: "DOLOR SIT AMET."
      })]
    })));
    const stage = _react.screen.getByTestId('stage');
    const nodes = getNodes(stage);
    const width = (0, _measureText.default)(nodes, stage);
    (0, _vitest.expect)(width).toBe(270);
  });
  it('should account for different nodes', async () => {
    const _render = (0, _react.render)(_div2 || (_div2 = (0, _jsxRuntime.jsxs)("div", {
        "data-testid": "stage",
        style: baseStyle,
        children: ["Lorem ipsum ", (0, _jsxRuntime.jsx)("span", {
          style: baseStyle,
          children: "DOLOR SIT AMET."
        })]
      }))),
      rerender = _render.rerender;
    const stage = _react.screen.getByTestId('stage');
    const nodes = getNodes(stage);
    const width = (0, _measureText.default)(nodes, stage);

    // Set child
    rerender(_div3 || (_div3 = (0, _jsxRuntime.jsx)("div", {
      "data-testid": "stage",
      style: baseStyle,
      children: "Lorem ipsum DOLOR SIT AMET."
    })));
    const stage2 = _react.screen.getByTestId('stage');
    const nodes2 = getNodes(stage2);
    const width2 = (0, _measureText.default)(nodes2, stage2);
    (0, _vitest.expect)(width).toEqual(width2);
  });
  it('should call measureText on a properly configured canvas 2d context', () => {
    const mockedStyle = {
      fontSize: '20px',
      fontWeight: 'bold',
      fontStyle: 'italic',
      fontFamily: 'Arial'
    };
    (0, _react.render)((0, _jsxRuntime.jsx)("div", {
      "data-testid": "stage",
      style: mockedStyle,
      children: "Lorem ipsum"
    }));
    const stage = _react.screen.getByTestId('stage');
    const nodes = getNodes(stage);
    (0, _measureText.default)(nodes, stage);

    // The width calculation depends on the modified canvas context
    const getContextSpy = HTMLCanvasElement.prototype.getContext;
    const context = getContextSpy.mock.results[0].value;
    (0, _vitest.expect)(context.measureText).toHaveBeenCalledTimes(1);
    (0, _vitest.expect)(context.font).toBe('bold italic 20px Arial');
  });
  it('should account for letter spacing styles', async () => {
    const _render2 = (0, _react.render)(_div4 || (_div4 = (0, _jsxRuntime.jsx)("div", {
        "data-testid": "stage",
        style: baseStyle,
        children: "Lorem ipsum"
      }))),
      rerender = _render2.rerender;
    const stage = _react.screen.getByTestId('stage');
    const nodes = getNodes(stage);
    const width = (0, _measureText.default)(nodes, stage);
    (0, _vitest.expect)(width).toBe(110); // default mocked width (text.length * 10)

    // Set letterSpacing
    rerender((0, _jsxRuntime.jsx)("div", {
      "data-testid": "stage2",
      style: {
        ...baseStyle,
        letterSpacing: '5px'
      },
      children: "Lorem ipsum"
    }));
    const stage2 = _react.screen.getByTestId('stage2');
    const nodes2 = getNodes(stage2);
    const width2 = (0, _measureText.default)(nodes2, stage2);
    (0, _vitest.expect)(width2).toBe(165); // default mocked width + letterOffset (text.length * letterSpacing)
  });
});